/* OS- and machine-dependent stuff for SYS5 */

/*
	FILE: UNIX.io.c
	
	Routines:
		ioinit()
		iostop()
		asy_init()
		asy_stop()
		asy_speed()
		asy_output()
		asy_recv()
		dir()
	Written or converted by Mikel Matthews, N9DVG
	SYS5 added by Jere Sandidge, K4FUM
	
	If you want to use the select code, define SELECT in the makefile or
	in this file.
*/

#include <stdio.h>
#include <sys/types.h>
#include <termio.h>
#include <signal.h>
#include <sys/file.h>
#include <sys/dir.h>
#include <time.h>
#include <fcntl.h>
#include "global.h"
#include "mbuf.h"
#include "internet.h"
#include "iface.h"
#include "unix.h"
#include "cmdparse.h"

struct asy asy[ASY_MAX];

struct interface *ifaces;

struct termio mysavetty, savecon;
int	IORser[ASY_MAX];
/*
int	IOWser;
*/
char *ttbuf;


/* Called at startup time to set up console I/O, memory heap */
ioinit()
{
	struct termio ttybuf;
	extern int iostop();
	register unsigned i;
	unsigned grabcore();
	char *malloc();

	/* Interrupts use a special stack deep in data space.
	 * Calls to sbrk() (invoked by malloc when it needs more memory
	 * from the system) at interrupt time will fail because sbrk()
	 * will think that the stack has overwritten the heap. So
	 * grab all the memory we can now for the heap so that malloc
	 * won't have to call sbrk and alloc_mbuf() won't fail unnecessarily
	 * at interrupt time.
	 */
	(void)signal(SIGHUP, iostop);
	(void)signal(SIGINT, iostop);
	(void)signal(SIGQUIT, iostop);
	(void)signal(SIGTERM, iostop);
	ioctl(0, TCGETA, &ttybuf);
	savecon = ttybuf;
	ttybuf.c_lflag &= ~ECHO;
	ttybuf.c_lflag &= ~ICANON;
	ttybuf.c_cc[VTIME] = '\01';
	ttybuf.c_cc[VMIN] = '\0';
	ioctl(0, TCSETA, &ttybuf);
}
/* Called just before exiting to restore console state */
iostop()
{
	setbuf(stdout,NULLCHAR);
	while(ifaces != NULLIF){
		if(ifaces->stop != NULLFP)
			(*ifaces->stop)(ifaces->dev);
		ifaces = ifaces->next;
	}
	ioctl(0, TCSETA, &savecon);
	exit(0);
}

/* Initialize asynch port "dev" */
int slipisopen;
int
asy_init(dev,bufsize)
int16 dev;
unsigned bufsize;
{
	register struct asy *ap;
	register struct interface *if_asy;
	extern struct interface *ifaces;
	struct termio	mytty, sgttyb;
	ap = &asy[dev];
	/*
	printf("asy_init: called\n");
	*/

	if ( ap == NULL)
	{
		return(-1);
	}

	printf("asy_init: tty name = %s\n", ap->tty);
	

	if ((IORser[dev] = open (ap->tty, (O_RDWR), 0)) < 0)
	{
		perror ("Could not open device IORser");
		return (-1);
	}
/*
	if ((IOWser = open (ap->tty, (O_WRONLY), 0)) < 0)
	{
		perror ("Could not open device IOWser");
		return (-1);
	}
*/
 /* 
  * get the stty structure and save it 
  */

	if (ioctl(IORser[dev], TCGETA, &mysavetty) < 0)
	{
		perror ("ioctl failed on device");
		return (-1);
	}
 /* 
  * copy over the structure 
  */

	sgttyb = mysavetty;
	sgttyb.c_iflag = (IGNBRK | IGNPAR);
	sgttyb.c_oflag = 0;
	sgttyb.c_lflag = 0;
	sgttyb.c_cflag = (B9600 | CS8 | CREAD);
	sgttyb.c_cc[VTIME] = '\05';
	sgttyb.c_cc[VMIN] = '\0';
	if (ioctl(IORser[dev], TCSETAF, &sgttyb) < 0)
	{
		perror ("ioctl could not set parameters for IORser");
		return (-1);
	}
/*
	if (ioctl (IOWser, TIOCSETP, &sgttyb) < 0)
	{
		perror ("ioctl could not set parameters for IOWser");
		return (-1);
	}
*/

	return (0);
}

int
asy_stop(dev)
int dev;
{
}

/* Set asynch line speed */
int
asy_speed(dev,speed)
int dev;
int speed;
{
	register int divisor;
	struct termio sgttyb;
	int i_state;

	if(speed == 0 || dev >= nasy)
		return(-1);
	/*
	printf("asy_speed: Setting speed for device %d to %d\n",dev, speed);
	*/
	asy[dev].speed = speed;

	if (ioctl(IORser[dev], TCGETA, &sgttyb) < 0)
	{
		perror ("ioctl could not set parameters");
		return (-1);
	}
	sgttyb.c_cflag &= ~CBAUD;
	switch(speed)
	{
		case 0:
			sgttyb.c_cflag |= B0;
			break;
		case 50:
			sgttyb.c_cflag |= B50;
			break;
		case 75:
			sgttyb.c_cflag |= B75;
			break;
		case 110:
			sgttyb.c_cflag |= B110;
			break;
		case 134:
			sgttyb.c_cflag |= B134;
			break;
		case 150:
			sgttyb.c_cflag |= B150;
			break;
		case 200:
			sgttyb.c_cflag |= B200;
			break;
		case 300:
			sgttyb.c_cflag |= B300;
			break;
		case 600:
			sgttyb.c_cflag |= B600;
			break;
		case 1200:
			sgttyb.c_cflag |= B1200;
			break;
		case 1800:
			sgttyb.c_cflag |= B1800;
			break;
		case 2400:
			sgttyb.c_cflag |= B2400;
			break;
		case 4800:
			sgttyb.c_cflag |= B4800;
			break;
		case 9600:
			sgttyb.c_cflag |= B9600;
			break;
		/*
		case EXTA:
			sgttyb.sg_ispeed = sgttyb.sg_ospeed = EXTA;
			break;
		case EXTB:
			sgttyb.sg_ispeed = sgttyb.sg_ospeed = EXTB;
			break;
		*/
		default:
			printf("asy_speed: Unknown speed (%d)\n", speed);
			break;
	}
/*
	printf("speed = %d\n", sgttyb.sg_ispeed);
	*/
	if (ioctl(IORser[dev], TCSETA, &sgttyb) < 0)
	{
		perror ("ioctl could not set parameters for IORser");
		return (-1);
	}
/*
	if (ioctl (IOWser, TIOCSETP, &sgttyb) < 0)
	{
		perror ("ioctl could not set parameters for IOWser");
		return (-1);
	}
*/
	return(0);
}
/* Send a buffer to serial transmitter */
asy_output(dev,buf,cnt)
unsigned dev;
char *buf;
unsigned short cnt;
{
	/*
	printf("asy_output called. dev = %x, cnt = %d\n", dev, cnt);
	printf("buf=%s\n", buf);
	fflush(stdout);
	*/
	
	if(dev >= nasy)
		return(-1);
	if ( write(IORser[dev], buf, cnt) < cnt)
	{
		perror("asy_output");
		printf("asy_output: error in writing to device %d\n", dev);
		return(-1);
	}
	return(0);
}
/* Receive characters from asynch line
 * Returns count of characters read
 */
unsigned
asy_recv(dev,buf,cnt)
int dev;
char *buf;
unsigned cnt;
{
	unsigned tot,n;
	int r;
	long amount;

#ifdef SELECT
	int	mask;
	int	writemask;
	int	ok;
	struct timeval timeout;
	timeout.tv_sec = 0;
	timeout.tv_usec = 35;
	mask = (1<<IORser[dev]);
	writemask = (1<<IORser[dev]);
	ok = 0;
	tot = 0;
	ok = select(mask, &mask, 0, 0, &timeout);
	if ( mask & (1<<IORser[dev]))
	{
		tot = read(IORser[dev], buf, cnt);
	}
	return (tot);
#else
	tot = 0;
	amount = 0;
	r = read(IORser[dev], buf, cnt);
	if (r == -1) {
		perror("asy_output");
		printf("asy_output: error in writing to device %d\n", dev);
		return(0);
	}
	tot = (unsigned int) r;
	return (tot);
#endif SELECT
}
/* Create a directory listing in a temp file and return the resulting file
 * descriptor. If full == 1, give a full listing; else return just a list
 * of names.
 *
 * This function is very dependent on the workings of Aztec standard I/O;
 * it uses their mechanism for generating and deleting temporary files.
 */
FILE *
dir(path,full)
char *path;
int full;
{
	FILE *fp;
	char cmd[1024];
	char filename[13];		/* instance of filename */
	char name[L_tmpnam], *tmpnam(), *malloc();
	tmpnam(name);

	if ( path[0] == '\0' )
	{
		path = ".";
		/*
		printf("Path = %s\n", path);
		*/
	}
	/*
	printf("DIR: tempname = %s, path = %s\n",name, path);
	*/

	if(full){
		sprintf(cmd,"ls -l %s > %s",path,name);
		system(cmd);
	} else {
		sprintf(cmd, "ls %s > %s", path, name);
		system(cmd);
	}
	if( (fp = fopen(name,"r")) == NULL)
	{
		perror("fopen");
	}
	/* Set up the magic cookies inside the file structure so that the
	 * temporary file gets deleted later when the file is closed
	 */
	(void)unlink(name);
	return (fp);
}
asy_ioctl(interface, argc, argv)
struct interface *interface;
int	argc;
char	*argv[];
{
	if (argc < 1) {
		printf("%d\r\n", asy[interface->dev].speed);
		return 0;
	}
	return asy_speed(interface->dev, atoi(argv[0]));
}
